//+------------------------------------------------------------------+
//|                                                   macd cross.mq4 | 
//|                                            original by Eli Hayun |
//|                                             mod by mrtools       |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1  clrBlue
#property indicator_color2  clrRed

//
//
//
//
//

extern ENUM_TIMEFRAMES    TimeFrame       = PERIOD_CURRENT;
extern int                FastEma         = 12;
extern int                SlowEma         = 26;
extern int                SignalPeriod    = 9;
extern ENUM_APPLIED_PRICE Price           = PRICE_CLOSE;
extern bool               alertsOn        = false;    // Turn alerts on
extern bool               alertsOnCurrent = true;     // Alerts on currnt (still opened) bar
extern bool               alertsMessage   = true;     // Alerts should show pop-up message
extern bool               alertsSound     = false;    // Alerts should play alert sound
extern bool               alertsPushNotif = false;    // Alerts should send push notification
extern bool               alertsEmail     = false;    // Alerts should send email
extern double             ArrowGapUp      = 0.5;      // Gap for arrow up        
extern double             ArrowGapDn      = 0.5;      // Gap for arrow down
extern bool               ArrowOnFirst    = false;     // Arrow on first bars

//
//
//
//
//

double CrossUp[];
double CrossDn[];
double trend[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{

   IndicatorBuffers(3);  
   SetIndexBuffer(0, CrossUp); SetIndexStyle(0, DRAW_ARROW); SetIndexArrow(0, 233);
   SetIndexBuffer(1, CrossDn); SetIndexStyle(1, DRAW_ARROW); SetIndexArrow(1, 234);
   SetIndexBuffer(2, trend);
   
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame==-99;
   TimeFrame         = MathMax(TimeFrame,_Period);      
         
   IndicatorShortName(timeFrameToString(TimeFrame)+  " Osma Cross ");
return(0);
}

int deinit()  {   return(0);  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { CrossUp[0] = limit+1; return(0); }


   //
   //
   //
   //
   //
   if (TimeFrame==Period())
   {
     for(i=limit; i>=0; i--)
     {
         double macdMain = iOsMA(NULL,0,FastEma,SlowEma,SignalPeriod,Price,i);
      
         trend[i] = trend[i+1];
         if (macdMain > 0) trend[i] = 1;
         if (macdMain < 0) trend[i] =-1;
         
         //
         //
         //
         //
         //
      
         CrossUp[i] = EMPTY_VALUE;
         CrossDn[i] = EMPTY_VALUE;
         if (trend[i]!= trend[i+1])
         if (trend[i] == 1)
               CrossUp[i] = Low[i]  - iATR(NULL,0,15,i)*ArrowGapUp;
         else  CrossDn[i] = High[i] + iATR(NULL,0,15,i)*ArrowGapDn; 
      }
      
      //
      //
      //
      //
      //
      
      if (alertsOn)
      {
        if (alertsOnCurrent)
             int whichBar = 0;
        else     whichBar = 1;
        if (trend[whichBar] != trend[whichBar+1])
        {
           if (trend[whichBar] == 1) doAlert(whichBar,"buy");
           if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
        }         
      }  
      return(0);
      }
      
      //
      //
      //
      //
      //
  
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
      for (i=limit;i>=0; i--)
      {
         int y = iBarShift(NULL,TimeFrame,Time[i]);
         int x = y;
         if (ArrowOnFirst)
               {  if (i<Bars-1) x = iBarShift(NULL,TimeFrame,Time[i+1]);          }
         else  {  if (i>0) x = iBarShift(NULL,TimeFrame,Time[i-1]); else x = -1;  }
              CrossUp[i] = EMPTY_VALUE;
              CrossDn[i] = EMPTY_VALUE;
              if (x!=y)
              {
                CrossUp[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,FastEma,SlowEma,SignalPeriod,Price,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsPushNotif,alertsEmail,ArrowGapUp,ArrowGapDn,0,y);
                CrossDn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,FastEma,SlowEma,SignalPeriod,Price,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsPushNotif,alertsEmail,ArrowGapUp,ArrowGapDn,1,y);
              }
      }
return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(_Period)+" Osma ",doWhat);
          if (alertsMessage)    Alert(message);
          if (alertsPushNotif)  SendNotification(message);
          if (alertsEmail)      SendMail(StringConcatenate(Symbol()," Osma "),message);
          if (alertsSound)      PlaySound("alert2.wav");
     }
}